#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/7/20  WeiYanfeng
    提供了 WinSV 需要一些公共函数。

-------- 依赖包
pip install psutil
pip install import_file

"""

import sys
from weberFuncs import PrintTimeMsg, PrettyPrintStr


def ReadTailLines(sFileName, iNum=5):
    # 读取文件尾部几行
    import linecache
    linecache.checkcache(sFileName)
    lsText = linecache.getlines(sFileName)[-iNum:]
    linecache.clearcache()
    return lsText


def TerminateByPid(pid):
    # 根据 pid 结束进程
    import psutil
    # process = psutil.Popen('TASKKILL /PID '+str(pid)+' /F')
    # process = os.popen('TASKKILL /PID '+str(pid)+' /F')
    # PrintTimeMsg("TerminateByPid.terminate(%s)=%s=" % (pid,str(process)))
    # return
    try:
        p = psutil.Process(pid)
        sCmd = p.cmdline()
        ret = p.terminate()
        # PrintTimeMsg("TerminateByPid.terminate(%s)=%s=" % (pid,ret))
        ret = p.wait(timeout=3)
        # PrintTimeMsg("TerminateByPid.wait(%s)=%s=" % (pid,ret))
        PrintTimeMsg("TerminateByPid(pid=%s)=(%s)" % (pid, sCmd))
    except psutil.NoSuchProcess as e:
        PrintTimeMsg("TerminateByPid(pid=%s).e=%s" % (pid, str(e)))


def GetSourcePath(srcfile):
    """
        取指定代码文件所在目录的绝对路径
    """
    import os
    import os.path
    if srcfile:
        sDir = os.path.dirname(os.path.realpath(srcfile))
        lsDir = sDir.split(os.sep)
        sDir = os.sep.join(lsDir[:])
        return sDir+os.sep
    else:
        PrintTimeMsg("Please use GetSourcePath(__file__)! Exit!")
        sys.exit(-1)


def GetAbsPathFmLocal(srcfile, sLocalPath=".", sFName=""):
    import os, inspect
    sAbsPath = os.path.realpath(os.path.abspath(os.path.join(
        os.path.split(GetSourcePath(srcfile))[0],sLocalPath)))
    # os.getcwd()
    # inspect.getfile( inspect.currentframe())
    sAbsPath += os.sep
    if sFName:
        return sAbsPath+sFName
    return sAbsPath


def ImportPythonClassOrVar(sPythonFileName, sClassOrVar):
    # 从python源码文件动态import
    try:
        from import_file import import_file
        imp = import_file(sPythonFileName)
        return getattr(imp,sClassOrVar)
    except Exception as e:
        import traceback
        traceback.print_exc()
        PrintTimeMsg('ImportPythonClassOrVar.Exception.e=(%s)' % (str(e)))


def LoadWinSVConfigFmDict(srcfile, dictConfigByGroupId, sLongIdStr):
    # 从 dictConfigByGroupId 加载WinSV配置参数到列表
    #   sLongIdStr 英文逗号分隔的<groupId.programId>
    #   单项中，若 programId 不存在，则包含整个程序组
    # 正常返回 命令行程序配置参数列表，出错返回空列表
    lsReturn = []
    for sId in sLongIdStr.split(','):
        if sId == '':  continue
        groupId,cSep,programId = sId.partition('.')
        if groupId == '':  #
            PrintTimeMsg("LoadWinSVConfigFmDict.sLongIdStr=(%s)Error!" % (sLongIdStr))
            return []
        else:
            dictGroup = dictConfigByGroupId.get(groupId, {})
            if dictGroup:
                def getDictFmGrp(programId):
                    dictProgram = dictGroup.get(programId,{})
                    dictParam = {}
                    dictParam['groupId'] = groupId
                    dictParam['programId'] = programId
                    dictParam['cmdExec'] = dictProgram.get('cmdExec','')
                    dictParam['cmdTitle'] = dictProgram.get('cmdTitle','')
                    # dictParam['shellPopen'] = dictProgram.get('shellPopen',False)

                    def getParamFmGrp(sKey):
                        sValue = dictProgram.get(sKey,'')
                        if sValue=='':  sValue = dictGroup.get(sKey,'')
                        return sValue

                    def getAbsPath(sDir):
                        if sDir.startswith('.'):
                            sDir = GetAbsPathFmLocal(srcfile,sDir)
                        return sDir
                    dictParam['workDir'] = getAbsPath(getParamFmGrp('workDir'))
                    dictParam['logDir'] = getAbsPath(getParamFmGrp('logDir'))
                    err2out = getParamFmGrp('err2out')
                    if err2out == '':  err2out = False
                    dictParam['err2out'] = err2out

                    shellPopen = getParamFmGrp('shellPopen')
                    if shellPopen == '':  shellPopen = False
                    dictParam['shellPopen'] = shellPopen

                    return dictParam
                if programId != '':
                    lsReturn.append(getDictFmGrp(programId))
                else:
                    for k,v in dictGroup.items():
                        if type(v) == dict:
                            lsReturn.append(getDictFmGrp(k))
    return lsReturn


def LoadWinSVConfigFmFile(srcfile, sPythonFileName, sDictVarName, sLongIdStr):
    # 从指定 Python 源码文件中加载WinSV配置
    dictConfigByGroupId = ImportPythonClassOrVar(sPythonFileName, sDictVarName)
    return LoadWinSVConfigFmDict(srcfile, dictConfigByGroupId, sLongIdStr)


def testMain():
    print(PrettyPrintStr(
        LoadWinSVConfigFmFile(__file__, 'demoSettingsWinSV.py',
                              'gDictConfigByGroupId',
                              'groupExample.programQQ,groupExample.programJC,')))
    return
    # from demoSettingsWinSV import gDictConfigByGroupId
    from import_file import import_file
    # gDictConfigByGroupId= ImportModuleClass('demoSettingsWinSV','gDictConfigByGroupId')
    imp = import_file('demoSettingsWinSV.py')
    PrintTimeMsg(dir(imp))
    gDictConfigByGroupId = getattr(imp,'gDictConfigByGroupId')
    PrintTimeMsg(gDictConfigByGroupId)
    PrintTimeMsg(PrettyPrintStr(LoadWinSVConfigFmDict(
        __file__,gDictConfigByGroupId,
        'groupExample.programQQ,groupExample.programJC,')))
    pass

# --------------------------------------
if __name__=='__main__':
    testMain()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__createTime__ = "2017/8/16 11:57"
__author__ = "WeiYanfeng"
__email__ = "weber.juche@gmail.com"
__version__ = "0.0.1"
        
~~~~~~~~~~~~~~~~~~~~~~~~
程序单元功能描述
测试sleep
~~~~~~~~~~~~~~~~~~~~~~~~
# 依赖包 Package required
# pip install weberFuncs

"""
import sys
from weberFuncs import PrintTimeMsg, PrintAndSleep


def mainRunSleep(iSeconds):
    iLoopCnt = 0
    while True:
        PrintAndSleep(iSeconds, 'mainRunSleep.iLoopCnt=%s' % iLoopCnt)
        iLoopCnt += 1
        if iLoopCnt >= 3:
            break
    PrintTimeMsg('mainRunSleep.End!')


# --------------------------------------
if __name__ == '__main__':
    iSeconds = 5
    mainRunSleep(iSeconds)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__createTime__ = "2017/8/16 11:31"
__author__ = "WeiYanfeng"
__email__ = "weber.juche@gmail.com"
__version__ = "0.0.1"
        
~~~~~~~~~~~~~~~~~~~~~~~~
程序单元功能描述
重复检查某应用程序（子命令）标准输出判断其运行状态，若发现吊死则重启。
~~~~~~~~~~~~~~~~~~~~~~~~
# 依赖包 Package required
# pip install weberFuncs

"""
import sys
from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep, PrintInline
from weberFuncs import GetTimeInteger
import time
import platform
import subprocess
# import psutil
import threading
from .winsvFuncs import TerminateByPid


class CCheckStdoutRestart:
    def __init__(self, iShouldOutInSeconds, iShouldEndInSeconds=0, sCoding='gbk'):
        self.iShouldOutInSeconds = iShouldOutInSeconds  # 应在N秒内有输出
        self.iShouldEndInSeconds = iShouldEndInSeconds  # 应在N秒内执行完
        self.sCoding = sCoding  # 输出结果解码
        PrintTimeMsg('CCheckStdoutRestart.iShouldOutInSeconds=(%s)' % self.iShouldOutInSeconds)
        PrintTimeMsg('CCheckStdoutRestart.iShouldEndInSeconds=(%s)' % self.iShouldEndInSeconds)
        PrintTimeMsg('CCheckStdoutRestart.sCoding=(%s)' % self.sCoding)

        self.oProcess = None  # 子进程对象
        self.iTmSubCmdStart = 0 # 获取子进程启动时间
        self.iTmLastStdout = 0  # 获取子进程上次标准输出的时间
        self.isLinux = platform.system() == 'Linux'
        # ON_POSIX = 'posix' in sys.builtin_module_names

    def __handleSubCmdStdout(self, sCmd):
        # 子线程处理子命令标准输出
        while True:
            if self.oProcess:
                for line in iter(self.oProcess.stdout.readline, b''):
                    self.iTmLastStdout = GetTimeInteger()
                    if self.sCoding:  # 从gbk解码到unicode
                        sLine = line.decode(self.sCoding)
                    else:
                        sLine = line
                    sLine = sLine.strip()
                    # PrintInline("   %s" % sLine)  # 避免在Win10下Wintail不换行
                    PrintInline("   %s\n" % sLine)
                self.oProcess.stdout.close()
                PrintTimeMsg('ChkCmd(%s).pid=(%s).End <<<' % (sCmd, self.oProcess.pid))
                self.oProcess = None
            else:
                time.sleep(0.001)  #

    def __startSubCmdByPopen(self, sCmd):
        # 启动子命令进程
        self.oProcess = subprocess.Popen(sCmd,
                                         stdin=subprocess.PIPE,
                                         stderr=subprocess.STDOUT,
                                         stdout=subprocess.PIPE,
                                         bufsize=1,
                                         close_fds=self.isLinux)  # ON_POSIX
        # 两个包的 Popen 函数效果一样。
        # self.oProcess = psutil.Popen(sCmd,
        #                              stdin=subprocess.PIPE,
        #                              stderr=subprocess.STDOUT,
        #                              stdout=subprocess.PIPE,
        #                              bufsize=1,
        #                              close_fds=self.isLinux)  # ON_POSIX
        PrintTimeMsg('Popen(%s).pid=(%s).Begin >>>' % (sCmd, self.oProcess.pid))
        self.iTmSubCmdStart = GetTimeInteger()
        self.iTmLastStdout = self.iTmSubCmdStart

    def LoopWaitAndExec(self, sCmd):
        # 启动线程监管子进程运行
        PrintTimeMsg("LoopWaitAndExec.sCmd=(%s)" % sCmd)
        oThread = threading.Thread(target=self.__handleSubCmdStdout, args=(sCmd,))
        oThread.daemon = True  # thread dies with the program
        oThread.start()
        # 主线程，负责启动子进程
        while True:
            self.__startSubCmdByPopen(sCmd)
            while self.oProcess:
                tmNow = GetTimeInteger()
                bQuit = tmNow - self.iTmLastStdout > self.iShouldOutInSeconds
                if not bQuit:
                    if self.iShouldEndInSeconds > 0:
                        bQuit = tmNow - self.iTmSubCmdStart > self.iShouldEndInSeconds
                if bQuit:
                    TerminateByPid(self.oProcess.pid)
                else:
                    time.sleep(0.001)


def mainCCheckStdoutRestart():
    o = CCheckStdoutRestart(10, 8)
    # o.LoopWaitAndExec('ping juchecar.com')  # ok
    # o.LoopWaitAndExec('ping juchecar.com -t')  # ok
    # o.LoopWaitAndExec(['ping', 'juchecar.com', '-t'])  # ok

    o.LoopWaitAndExec('python RunSleep.py')


# --------------------------------------
if __name__ == '__main__':
    mainCCheckStdoutRestart()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
2019/7/31  WeiYanfeng

简化 CWinSupervisor.py ，直接传入 dictConfigByGroupId ，以便于直接启动多个进程。
"""

import sys
import os

from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep
from .winsvFuncs import LoadWinSVConfigFmDict
from .CWinSVProgram import CWinSVProgram


class CRunSupervisor:
    def __init__(self, srcfile, dictConfigByGroupId, sLongIdStr):
        # PrintTimeMsg("CRunSupervisor.__init__.srcfile=%s=" % srcfile)
        # PrintTimeMsg("CRunSupervisor.__init__.sLongIdStr=%s=" % sLongIdStr)
        self.sStartCWD = os.getcwd()
        # PrintTimeMsg("CRunSupervisor.__init__.sStartCWD=%s=" % self.sStartCWD)
        self.lsProgram = LoadWinSVConfigFmDict(srcfile, dictConfigByGroupId, sLongIdStr)
        iProgramCnt = len(self.lsProgram)
        if iProgramCnt == 0:
            PrintTimeMsg("CRunSupervisor.__init__.iProgramCnt=(%d)Error EXIT!" % iProgramCnt)
            sys.exit(-1)
        PrintTimeMsg("CRunSupervisor.__init__.iProgramCnt=(%d)!" % iProgramCnt)

        self.iCheckIntervalSeconds = 60

        iCmdCnt = 0
        self.lsWinSV = []
        for dictParam in self.lsProgram:
            oWinSV = CWinSVProgram(dictParam)
            self.lsWinSV.append(oWinSV)
            # PrintTimeMsg("  lsWinSV[%.2d]=(%s)!" % (iCmdCnt, oWinSV.sCmdExec))
            iCmdCnt += 1

    def __del__(self):
        pass

    def LoopAndWatchPrograms(self):
        iLoopCnt = 0
        while True:
            os.chdir(self.sStartCWD)  # 回到启动时的目录
            iStart, iStop = 0, 0
            for oWinSV in self.lsWinSV:
                iChg = oWinSV.CheckAndRunOneCmd(iLoopCnt)
                if iChg > 0:
                    iStart += 1
                if iChg < 0:
                    iStop += 1
            if iStart > 0 or iStop > 0:
                PrintTimeMsg("  >>(%s)#%d.iStart=%s,iStop=%s!" % (
                    self.sStartCWD, iLoopCnt, iStart, iStop))
            else:
                PrintAndSleep(self.iCheckIntervalSeconds,
                              "  >>(%s)#%d" % (self.sStartCWD, iLoopCnt),
                              iLoopCnt % 10 == 0)
            iLoopCnt += 1


def mainCRunSupervisor(srcfile, dictConfigByGroupId, sLongIdStr):
    ws = CRunSupervisor(srcfile, dictConfigByGroupId, sLongIdStr)
    ws.LoopAndWatchPrograms()

# --------------------------------------
if __name__ == '__main__':
    dictConfigByGroupId = {
        'group_One': {  # 单一程序组
            'logDir': '../log',
            'workDir': './',
            'err2out': True,
            'ping_qq': {
                'cmdExec': 'ping www.qq.com',
                'cmdTitle': 'ping_qq test',
            },
        },
    }
    sLongIdStr = 'group_One'
    mainCRunSupervisor(__file__, dictConfigByGroupId, sLongIdStr)



#!/usr/local/bin/python
# -*- coding:utf-8 -*-

import codecs
import os
import sys

try:
    from setuptools import setup  # 打包的用的setup必须引入
except ImportError as e:
    from distutils.core import setup


def loadLongDesc(fname):
    """
    定义一个read方法，用来读取目录下的长描述
    我们一般是将README文件中的内容读取出来作为长描述，这个会在PyPI中你这个包的页面上展现出来，
    你也可以不用这个方法，自己手动写内容即可，
    PyPI上支持.rst格式的文件。暂不支持.md格式的文件，<BR>.rst文件PyPI会自动把它转为HTML形式显示在你包的信息页面上。
    """
    # return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()
    # WeiYF.20170605 为避免 pip install 时出现 UnicodeDecodeError 暂时直接使用简单串
    return "weberWinSV"

setup(
    name="weberWinSV",  # 包的名字
    packages=["weberWinSV"],  # 包含的包，可以多个，这是一个列表
    version="0.0.21",  # 当前包的版本，这个按你自己需要的版本控制方式来

    keywords="Supervisor for Windows weber",  # 关于当前包的一些关键字，方便PyPI进行分类。
    author="Weber Juche",  # 包的作者
    author_email="weber.juche@gmail.com",  # 作者的邮件地址
    url="https://github.com/Weber-JC/PythonEggs/tree/master/weberWinSV",  # 这个包的项目地址
    license="MIT",  # 授权方式,

    # zip_safe=True,
    zip_safe=False,

    description= 'Supervisor for Windows wrote by Weber Juche.',
    # u"包含了常用的简单函数的共享库", #关于这个包的描述,这个会出现 PyPI 的列表页面

    long_description = "long_description",  # loadLongDesc("README.rst"),  # 参见loadLongDesc方法说明
    long_description_content_type='text/markdown',

    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    include_package_data=True,

    install_requires=[
        # WeiYF.20160719 为避免本地安装时强制更新，这里不列入依赖。
        # "requests",
    ], # 所依赖的PyPi包
    # dependency_links=['http://github.com/user/repo/tarball/master#egg=package-1.0'] # 非PyPi包
)

if __name__ == '__main__':
    # print(loadLongDesc("README.rst"))
    pass



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
Created on 2016-7-19
@author: Weber Juche

WeberWinSV 配置参数说明及示例。
适用于 CWinSupervisor 和 CWebSupervisor 两个工具。

"""

gDictConfigByGroupId = {
    # WeberWinSV 配置参数，是新版程序管理设计的核心。

    # workDir 和 logDir 要附带最后路径分隔符，Windows路径要写两个反斜杠
    # programId=cmdProgram标识，用于标识该命令行程序。参与生成日志文件、pid文件等。

    '<groupId>': {  # 以 groupId 为键值，内容是程序组的配置参数字典。
        # 管理程序也以程序组(groupId=程序组标识)为单位进行管理，也支持同时多个。
        # 在 groupId 编码中不能有英文逗号和点号，可以通过下划线加上
        #   dev=开发 和 run=运行 支持两种环境。
        # 为了对参数进行说明，尖括号开始的键值是示例，复制修改时应予以保留。
        'groupTitle': '程序组说明',
        'workDir': '该程序组缺省工作目录',
        'logDir': '该程序组缺省日志目录',
        'err2out': 'True,合并标准错误日志到标准输出，该程序组的缺省设置',
        '<programId>': {  # 以 programId 为键值，内容是 cmdProgram 的配置参数字典
            'cmdExec': 'cmdProgram的命令串，请首先在命令行进行测试',
            'cmdTitle': '当前cmdProgram标题，文字描述',
            'shellPopen': 'True/False,调用Popen参数shell参数，缺省是False',
            'workDir': '当前cmdProgram的工作目录',
            'logDir': '当前cmdProgram的日志输出目录',
            'err2out': 'True/False,合并标准错误日志到标准输出',
        },
    },
    'groupExample': {  # 例子程序组, 用于修改替换为实际的配置。
        'logDir': u'./test/log/',
        'workDir': u'./test/',
        # 'workDir': u'D:\\WeiYFGitSrc\\PythonProject\\WeberEgg\\GitHub_PythonEggs\\weberWinSV\\weberWinSV\\test\\',
        'err2out': True,
        'programJC': {  # 测试程序
            'cmdExec': 'ping juchecar.com',
            'cmdTitle': '测试ping JC',
        },
        'programQQ': {  # 测试程序
            'cmdExec': 'ping qq.com',
            'cmdTitle': '测试ping QQ',
        },
        'programQQt': {  # 测试程序
            'cmdExec': 'ping qq.com -t',
            'cmdTitle': '测试ping QQ -t',
        },
        'programDirC': {  # 测试程序
            'cmdExec': 'dir C:\\',
            'cmdTitle': '测试dir C:',
            'shellPopen': True,  # 不设置为True, 会出异常
        },
    },
}


